@@ -157,33 +157,22 @@ _______________
   (workaround: killing of java PyCharm process).
 """
 from abc import abstractmethod, ABC
+import os
+import pathlib
 from typing import Any, Dict, List, Optional, Sequence
 
-from ae.core import DEBUG_LEVEL_VERBOSE                                         # type: ignore
+from ae.core import DEBUG_LEVEL_VERBOSE, try_eval                               # type: ignore
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.0.12'
+__version__ = '0.0.14'
 
 
-FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper and for drop onto leave item button
-FLOW_PATH_TEXT_SEP = " / "           #: flow path separator for :meth:`~LiszDataMixin.flow_path_text` and drop enter
+NODE_FILE_PREFIX = 'node_'  #: file name prefix for node imports/exports
+NODE_FILE_EXT = '.txt'      #: file extension for node imports/exports
 
-HELP_TEXT = (
-    "Touch item name for to toggle selection.\n"
-    "\n"
-    "Filter un-/selected items with the eye\n"
-    "icons in the top right corner.\n"
-    "\n"
-    "Touch (-->) icon of item for to\n"
-    "display their sub-items.\n"
-    "\n"
-    "Edit focused item for to change name\n"
-    "or for to attach/remove a node.\n"
-    "\n"
-    "Change order by dragging item with the\n"
-    "rightmost icon.\n"
-)
+FLOW_PATH_ROOT_ID = 'ROOT'  #: needed for flow path jumper and for drop onto leave item button
+FLOW_PATH_TEXT_SEP = " / "  #: flow path separator for :meth:`~LiszDataMixin.flow_path_text` and drop enter
 
 
 LiszItem = Dict[str, Any]           #: node item data (nid) type
@@ -205,6 +194,7 @@ class LiszDataMixin(ABC):
     root_node: LiszNode             #: root of lisz data structure
     current_node_items: LiszNode    #: node item data of the current node / sub list (stored as app state via root_node)
     filtered_indexes: List[int]     #: indexes of the filtered/displayed items in the current node
+    # additional app-common attributes (never directly referenced/needed, KEEP for completeness/documentation).
     filter_selected: bool           #: True for to hide/filter selected node items
     filter_unselected: bool         #: True for to hide/filter unselected node items
 
@@ -212,10 +202,11 @@ class LiszDataMixin(ABC):
     debug_level: int                #: :attr:`~AppBase.debug_level`
     flow_id: str                    #: current attr:`flow id <ae.gui_app.MainAppBase.flow_id>`
     flow_path: List[str]            #: :attr:`flow path <ae.gui_app.MainAppBase.flow_path>` ref. current node
-    framework_win: Any              #: framework window
 
     _refreshing_data: bool = False                      #: DEBUG True while running :meth:`~.refresh_all` method
 
+    # abstract methods
+    # the gui_app methods are not really abstract (nor callable via super()) but declared here for to hide inspections
     @abstractmethod
     def call_method(self, method: str, *args, **kwargs) -> Any:
         """ call method of this instance (ae.gui_app.MainAppBase method). """
@@ -225,7 +216,7 @@ class LiszDataMixin(ABC):
         """ add debug sound on each state change/save (ae.gui_app.MainAppBase method). """
 
     @abstractmethod
-    def change_flow(self, ifi: str, popups_to_close: Sequence = (), **event_kwargs) -> bool:
+    def change_flow(self, new_flow_id: str, popups_to_close: Sequence = (), **event_kwargs) -> bool:
         """ change/switch flow id. """
 
     @abstractmethod
@@ -236,6 +227,7 @@ class LiszDataMixin(ABC):
     def play_sound(self, sound_name: str):
         """ play audio/sound file (ae.gui_app.MainAppBase method). """
 
+    # this a real abstract method (to be implemented by inheriting class)
     @abstractmethod
     def refresh_node_widgets(self):
         """ redraw/refresh the widgets representing the current node items/sub-nodes (GUI framework method). """
@@ -243,10 +235,10 @@ class LiszDataMixin(ABC):
     # helper methods
 
     def change_sub_node_sel(self, node: LiszNode, set_sel_to: bool):
-        """ change the selection of all the sub-leafs of the passed node to the specified value.
+        """ change the selection of all the sub-leaves of the passed node to the specified value.
 
-        :param node:            node of which to change the selection of the sub-item leafs.
-        :param set_sel_to:      True will only toggle the unselected sub-item leafs, False only the selected ones.
+        :param node:            node of which to change the selection of the sub-item leaves.
+        :param set_sel_to:      True will only toggle the unselected sub-item leaves, False only the selected ones.
         """
         for item in node:
             item.pop('sel', None)       # always first remove sel left-overs
@@ -274,7 +266,7 @@ class LiszDataMixin(ABC):
     def edit_validate(self, old_idx: int, new_id: Optional[str] = None, want_node: Optional[bool] = None) -> str:
         """ validate the user changes after adding a new item or editing an existing item.
 
-        :param old_idx:             index in the current node of the edited item or -1 if a new item got edited.
+        :param old_idx:             index in the current node of the edited item or -1 if a new item (to be added).
         :param new_id:              new/edited id string.
         :param want_node:           True if the new/edited item will have a sub-node, False if not.
         :return:                    empty string on successful validation, else error string or
@@ -317,6 +309,25 @@ class LiszDataMixin(ABC):
 
         return ""
 
+    def export_node(self, flow_path: List[str], file_path: str = ".") -> Optional[Exception]:
+        """ export node specified by the passed :paramref:`~export_node.flow_path` argument.
+
+        :param flow_path:   flow path of the node to export.
+        :param file_path:   folder to store the node data into (def=current working directory).
+        :return:            `None` if node got exported without errors, else the raised exception.
+        """
+        node = self.flow_path_node(flow_path)
+        file_name = NODE_FILE_PREFIX + (flow_key(flow_path[-1]) if flow_path else FLOW_PATH_ROOT_ID) + NODE_FILE_EXT
+
+        try:
+            # the alternative `os.makedirs(exist_ok=True)` has problems on POSIX with '..'
+            pathlib.Path(file_path).mkdir(parents=True, exist_ok=True)
+            with open(os.path.join(file_path, file_name), 'w') as file_handle:
+                file_handle.write(str(node))
+        except (FileExistsError, FileNotFoundError, OSError, ValueError) as ex:
+            return ex
+        return None
+
     def find_item_index(self, item_id: str, searched_node: Optional[LiszNode] = None) -> int:
         """ determine list index of the passed item id in the searched node or in the current node.
 
@@ -465,6 +476,31 @@ class LiszDataMixin(ABC):
                 idx = min(max(-1, delta), 0)
             self.change_flow(id_of_flow('focus', 'item', self.current_node_items[filtered_indexes[idx]]['id']))
 
+    def import_nodes(self, file_path: str = ".") -> List[str]:
+        """ import all nodes found in the documents app folder of this app into the currently displayed node.
+
+        :param file_path:   path to the folder wherein to store the node data (def=current working directory).
+        :return:            empty list if found node(s) got imported without errors,
+                            else the list of error strings prefixed with the node file path.
+        """
+        errors = list()
+        try:
+            for node_file in os.scandir(file_path):
+                file_name = node_file.name
+                if file_name.endswith(NODE_FILE_EXT) and node_file.is_file():
+                    node_name = "imported node " + file_name[len(NODE_FILE_PREFIX):-len(NODE_FILE_EXT)]
+                    err_msg = self.edit_validate(-1, node_name, want_node=True)
+                    if err_msg:
+                        errors.append(node_file.path + ":" + err_msg)
+                        continue
+                    with open(node_file.path) as file_handle:
+                        file_content = file_handle.read()
+                    # update the list instance that got already added by edit_validate() with the loaded node data
+                    self.current_node_items[0]['node'][:] = try_eval(file_content)
+        except (FileExistsError, FileNotFoundError, OSError, ValueError) as ex:
+            errors.append(str(ex))
+        return errors
+
     def item_by_id(self, item_id: str, searched_node: Optional[LiszNode] = None) -> LiszItem:
         """ search item in either the passed or the current node.
 
@@ -630,7 +666,7 @@ class LiszDataMixin(ABC):
         return True
 
     def on_item_sel_change(self, item_id: str, event_kwargs: dict) -> bool:
-        """ toggle, set or reset in the current node the selection of a leaf item or of the sub-leafs of a node item.
+        """ toggle, set or reset in the current node the selection of a leaf item or of the sub-leaves of a node item.
 
         :param item_id:         item id of the leaf to toggle selection for.
         :param event_kwargs:    event kwargs, containing a `set_sel_to` key with a boolean value, where
@@ -659,7 +695,7 @@ class LiszDataMixin(ABC):
         Used for to jump to node specified by the flow path text in the passed flow_id key.
 
         :param flow_path_text:  flow path text (identifying where to jump to).
-        :param event_kwargs:    event arguments (not used here).
+        :param event_kwargs:    event arguments (used for to reset flow id).
         :return:                True for to process/change flow.
         """
         flow_path = self.flow_path_from_text(flow_path_text)
@@ -723,7 +759,7 @@ class LiszDataMixin(ABC):
             if 'node' in item:
                 self.shrink_node_size(item['node'])
 
-    def sub_item_ids(self, item_id: str, leafs_only: bool, hide_sel_val: Optional[bool] = None,
+    def sub_item_ids(self, item_id: str, leaves_only: bool, hide_sel_val: Optional[bool] = None,
                      node: Optional[LiszNode] = None, sub_ids: Optional[List[str]] = None) -> List[str]:
         """ return item names of all items including their sub-node items (if exists).
 
@@ -731,7 +767,7 @@ class LiszDataMixin(ABC):
         the node item specified by the :paramref:`sub_item_ids.item_id` argument.
 
         :param item_id:         item id of the node item to be investigated (for to be deleted or de-/selected later).
-        :param leafs_only:      True if only include the leaf item ids of the sub-nodes of the node item.
+        :param leaves_only:     True if only include the leaf item ids of the sub-nodes of the node item.
         :param hide_sel_val:    pass False/True for to exclude un-/selected items from the returned list of ids.
                                 If None or not passed then all found items will be included.
         :param node:            searched node, use current node if not passed (used for recursive calls of this method).
@@ -745,14 +781,14 @@ class LiszDataMixin(ABC):
 
         item = self.item_by_id(item_id, node)
 
-        if not leafs_only and (hide_sel_val is None or bool(item.get('sel')) != hide_sel_val):
+        if not leaves_only and (hide_sel_val is None or bool(item.get('sel')) != hide_sel_val):
             sub_ids.append(item_id)
 
         node = item.get('node')
         if node is not None:
             for item in node:
                 if item.get('node'):
-                    self.sub_item_ids(item['id'], leafs_only, hide_sel_val=hide_sel_val, node=node, sub_ids=sub_ids)
+                    self.sub_item_ids(item['id'], leaves_only, hide_sel_val=hide_sel_val, node=node, sub_ids=sub_ids)
                 elif hide_sel_val is None or bool(item.get('sel')) != hide_sel_val:
                     sub_ids.append(item['id'])
 
